<?php
/**
 * @file
 * Actions that can be performed on a file
 */

/**
 * Implements hook_file_operations().
 *
 * Defines the operations that can be performed on selected files
 *
 * @param array $form_state
 *   The current state of the form in case we have to inject stuff
 */
function private_files_operations($form_state = array()) {
  $roles = user_roles(TRUE);
  $add_roles = array();
  $del_roles = array();
  foreach ($roles as $key => $value) {
    $add_roles['add_role-' . $key] = $value;
    $del_roles['del_role-' . $key] = $value;
  }

  $operations = array(
    'delete' => array(
      'label' => t('Delete the selected files'),
    ),
    'attach' => array(
      'label' => t('Attach selected files to selected node'),
      'callback' => 'private_files_mass_attach',
      'callback arguments' => array($form_state['values']['node']),
    ),
    'taxonomy' => array(
      'label' => ('Attach keywords to selected files'),
      'callback' => 'private_files_mass_taxonomy',
      'callback arguments' => array($form_state['values']['keywords']),
    ),
  );

  // These operations need to be t() wrapped as they get displayed
  // in the dropdown, unlike those above.
  if (count($roles)) {
    $operations += array(
      t('Add a role to the selected files') => array(
        'label' => $add_roles,
      ),
      t('Remove a role from the selected files') => array(
        'label' => $del_roles,
      ),
    );
  }

  // If the form has been posted, we need to insert the proper data for
  // role editing if necessary.
  if (!empty($form_state['submitted'])) {
    $operation_rid = explode('-', $form_state['values']['action']);
    $operation = $operation_rid[0];
    if ($operation == 'add_role' || $operation == 'del_role') {
      $rid = $operation_rid[1];
      if (user_access(PRIVATE_FILES_UPLOAD)) {
        $operations[$form_state['values']['action']] = array(
          'callback' => 'private_files_role_edit',
          'callback arguments' => array($operation, $rid),
        );
      }
      else {
        watchdog(
          'security',
          'Detected malicious attempt to alter private file.',
          array(), WATCHDOG_WARNING);
      }
    }
  }
  return $operations;
}

/**
 * Callback for mass role setting operation
 *
 * @param array $files
 *   files to process
 * @param string $operation
 *   the operation, adding or deleting the role
 * @param integer $rid
 *   the role ID
 *
 * @return boolean
 *   TRUE for success, FALSE for failure
 */
function private_files_role_edit($files, $operation, $rid) {
  module_load_include('inc', 'private_files', 'private_files.db');
  $ret = FALSE;
  foreach ($files as $fid) {
    if ($file = private_files_get_file($fid)) {
      if ($file->is_private) {
        if ($operation == 'add_role') {
          private_files_add_role($fid, $rid);
        }
        else {
          private_files_del_role($fid, $rid);
        }
        $ret = TRUE;
      }
      else {
        drupal_set_message(t('You cannot assign roles to public files.'), 'error');
      }
    }
    else {
      drupal_set_message(t('Invalid file ID.'), 'error');
    }
  }
  return $ret;
}

/**
 * Callback for mass file attach operation
 *
 * @param array $files
 *   files to process
 * @param string $node
 *   node ID and title as generated by autocomplete
 *
 * @return boolean
 *   TRUE unless the node is not found, then FALSE
 */
function private_files_mass_attach($files, $node) {
  global $user;
  module_load_include('inc', 'private_files', 'private_files.db');
  if (count($files) && preg_match('/\[nid:(\d+)\]/', $node, $matches)) {
    $nid = $matches[1];
    if ($node = node_load($nid)) {
      foreach ($files as $fid) {
        $file = private_files_get_file($fid);
        $filefield = array(
          'fid' => $file->fid,
          'filename' => $file->filename,
          'filepath' => 'system/private_files/' . $file->filename,
          'filesize' => $file->filesize,
          'filemime' => $file->filemime,
          'list' => 1,
          'uid' => $user->uid,
          'status' => 1,
          'timestamp' => time(),
        );
        switch ($file->filemime) {
          case 'image/jpeg':
          case 'image/gif':
          case 'image/png':
            // Don't use imagefield for private files
            // because imagecache will expose the derivatives.
            if ($file->is_private) {
              $node->field_file[] = $filefield;
            }
            else {
              $node->field_image[] = $filefield;
            }
            break;
          default:
            $node->field_file[] = $filefield;
        }
      }
      node_submit($node);
      node_save($node);
      return TRUE;
    }
    else {
      drupal_set_message(t('Cannot find node'), 'error');
      return FALSE;
    }
  }
}

/**
 * Callback for mass file taxonomy operation
 *
 * @param array $files
 *   files to process
 * @param string $keywords
 *   comma separated list of keywords to attach
 *
 * @return boolean
 *   TRUE
 */
function private_files_mass_taxonomy($files, $keywords) {
  module_load_include('inc', 'private_files', 'private_files.db');
  foreach ($files as $fid) {
    private_files_add_taxonomy($fid, $keywords);
  }
  return TRUE;
}

/**
 * Menu callback for node autocomplete
 */
function private_files_node_autocomplete($string) {
  $items = array();
  module_load_include('inc', 'private_files', 'private_files.db');
  $nodes = private_files_get_node_autocomplete($string);
  foreach ($nodes as $nid => $title) {
    $title = check_plain($title) . "[nid:$nid]";
    $items[$title] = $title;
  }
  drupal_json($items);
}
